#!@@PERL@@ @@PERLOPTS@@

# Copyright 1999, 2000, 2001 (c) Thomas Erskine <@@AUTHOR@@>
# See the COPYRIGHT file with the distribution.

# new-config - make a new config-dir as links to config-base
# $Id: new-config.pl,v 1.9 2001/08/28 15:22:24 remstats Exp $

# - - -   Configuration   - - -

use strict

# What is this program called, for error-messages and file-names
$main::prog = 'new-config';
# Where is the base config-dir
$main::config_base = '@@CONFIGDIR@@-base';
# Which things in config-base get linked here
my @links = ('alert-template-map', 'alert-templates', 'archives', 
	'availability', 'colors', 'oids', 'remotepings', 'rrds', 
	'scripts', 'times', );
# Which things should be copied instead
my @copies = ('alerts', 'alert-destination-map', 'discovery',
	'general', 'html', 'links', 'tools');
# Which things need to be real sub-directories
my @subdirs = ('customgraphs', 'datapages', 'hosts', 'views', 
	'view-templates', 'host-templates');

# - - -   Version History   - - -

(undef, $main::version) = split(' ', '$Revision: 1.9 $');

# - - -   Setup   - - -

use Getopt::Std;

my %opt = ();
getopts("d:f:h", \%opt);

if (defined $opt{'h'}) { &usage; } # no return
if (defined $opt{'f'}) { $main::config_base = $opt{'f'}; }
if (defined $opt{'d'}) { $main::debug = $opt{'d'}; } else { $main::debug = 0; }

unless ($#ARGV == 0) { &usage; }

$main::config_dir = shift @ARGV;

unless (-d $main::config_base) {
	&abort("$main::config_base is missing or isn't a directory");
}

if (-e $main::config_dir) { &abort("$main::config_dir exists"); }

# - - -   Mainline   - - -

mkdir $main::config_dir, 0755 or &abort("can't mkdir $main::config_dir: $!");
&debug("mkdir for $main::config_dir") if ($main::debug);

# Make the symlinks
my ($old, $new, $dir, $file, $data);
foreach $dir (@links) {
	$old = $config_base .'/'. $dir;
	$new = $config_dir .'/'. $dir;
	symlink $old, $new or
		&abort("can't symlink $old to $new: $!");
	&debug("  symlinked $dir from $old") if ($main::debug);
}

# Make the copies
foreach $file (@copies) {
	$old = $config_base .'/'. $file;
	$new = $config_dir .'/'. $file;

	open (OLD, "<$old") or &abort("can't open $old: $!");
	$data = join('', <OLD>) or &abort("can't read $old: $!");
	close (OLD) or &abort("can't close $old: $!");

	open (NEW, ">$new") or &abort("can't open $new: $!");
	print NEW $data or &abort("can't write $new: $!");
	close (NEW) or &abort("can't close $new: $!");
}

# Make the sub-drectories
foreach $dir (@subdirs) {
	$new = $config_dir .'/'. $dir;
	mkdir $new, 0755 or &abort("can't mkdir $new: $!");
	&debug("  mkdir for $dir") if ($main::debug);
}

# Make an (almost) empty groups file
$new = $config_dir .'/groups';
open (GROUPS, ">$new") or &abort("can't open $new: $!");
print GROUPS "# groups - the groups in the order you want to see them\n\n";
print GROUPS "# you probably want this one last\nRemstats\n";
close (GROUPS);

# Make the _remstats_ pseudo-host
$new = $config_dir .'/hosts/_remstats_';
open (REMSTATS, ">$new") or &abort("can't open $new: $!\n");
print REMSTATS <<"EOD_REMSTATS";
# hosts/_remstats_ - remstats self-monitoring
#
desc	the status of remstats itself
group	Remstats
tools	availability status
rrd	collector-log
rrd     collector-nt-status
rrd     collector-ping
rrd     collector-port
rrd     collector-snmp
#rrd    collector-snmp-route
rrd     collector-unix-status
EOD_REMSTATS
close (REMSTATS);

0;

#------------------------------------------------ usage ---
sub usage {
	print STDERR <<"EOD_USAGE";
$main::prog version $main::version
usage: $main::prog [options] new-config-dir
where options are:
    -d ddd  set debug level to 'ddd'
    -f fff  use 'fff' as config-base [$main::config_base]
    -h      show this text
EOD_USAGE
	exit 0;
}

#------------------------------------------------ debug ---
sub debug {
	my ($msg) = @_;
	print STDERR "DEBUG: $msg\n" if ($main::debug);
}

#----------------------------------------------- abort ---
sub abort {
	my ($msg) = @_;
	print STDERR "$main::prog: ABORT: $msg\n";
	exit 1;
}

#----------------------------------------------- error ---
sub error {
	my ($msg) = @_;
	print STDERR "ERROR: $msg\n";
}
